@@ -79,8 +79,8 @@ except ImportError as error1:
 
 __Conventions__  = 'CF-1.7'
 __author__       = 'David Hassell'
-__date__         = '2019-11-01'
-__version__      = '3.0.3'
+__date__         = '2019-11-08'
+__version__      = '3.0.4'
 __cf_version__   = cfdm.core.__cf_version__
 
 from distutils.version import LooseVersion
@@ -188,8 +188,8 @@ from .cellmethod           import CellMethod
 from .cellmeasure          import CellMeasure
 from .domainancillary      import DomainAncillary
 from .domainaxis           import DomainAxis
-from .field                import Field #, FieldList
 from .fieldancillary       import FieldAncillary
+from .field                import Field
 from .data                 import (Data,
                                    FilledArray,
                                    GatheredArray,
