@@ -1975,7 +1975,7 @@ def inspect(self):
     
     if hasattr(self, '__dict__'):
         for key, value in sorted(self.__dict__.items()):
-            out.append('%s: %s' % (key, repr(value)))
+            out.append('{}: {!r}'.format(key, value))
         
     print('\n'.join(out))
 
@@ -2399,7 +2399,6 @@ def default_netCDF_fillvals():
     return netCDF4.default_fillvals
 
 
-
 def _DEPRECATION_ERROR(message='', version='3.0.0'):
     raise DeprecationError("{}".format(message))
 
