@@ -1,20 +1,4 @@
-from copy      import deepcopy
-from functools import partial as functools_partial
-from re        import escape  as re_escape
-from re        import match   as re_match
-from re        import findall as re_findall
-from re        import search as re_search
-from netCDF4   import default_fillvals as _netCDF4_default_fillvals
-from operator  import truediv  as truediv
-from operator  import itruediv as itruediv
-from re        import compile as re_compile
-
-
-from numpy import array       as numpy_array
-from numpy import result_type as numpy_result_type
-from numpy import size        as numpy_size
-from numpy import vectorize   as numpy_vectorize
-
+from numpy import size as numpy_size
 
 from . import PropertiesData
 
@@ -1422,6 +1406,70 @@ The output units are '1' (nondimensionsal).
         return v
 
     
+    def flatten(self, axes=None, inplace=False):
+        '''Flatten axes of the data
+
+    Any subset of the axes may be flattened.
+
+    The shape of the data may change, but the size will not.
+
+    The flattening is executed in row-major (C-style) order. For
+    example, the array ``[[1, 2], [3, 4]]`` would be flattened across
+    both dimensions to ``[1 2 3 4]``.
+
+    .. versionaddedd:: 3.0.2
+
+    .. seealso:: `insert_dimension`, `flip`, `swapaxes`, `transpose`
+
+    :Parameters:
+   
+        axes: (sequence of) int or str, optional
+            Select the axes.  By default all axes are flattened. The
+            *axes* argument may be one, or a sequence, of:
+    
+              * An internal axis identifier. Selects this axis.
+            ..
+    
+              * An integer. Selects the axis coresponding to the given
+                position in the list of axes of the data array.
+    
+            No axes are flattened if *axes* is an empty sequence.
+    
+        inplace: `bool`, optional
+            If True then do the operation in-place and return `None`.
+    
+    :Returns:
+
+            The construct with flattened data, or `None` if the
+            operation was in-place.
+
+    **Examples**
+
+    >>> f.shape
+    (1, 2, 3, 4)
+    >>> f.flatten().shape
+    (24,)
+    >>> f.flatten([1, 3]).shape
+    (1, 8, 3)
+    >>> f.flatten([0, -1], inplace=True)
+    >>> f.shape
+    (4, 2, 3)
+
+        '''
+        v = super().flatten(axes, inplace=inplace)
+        if inplace:
+            v = self
+            
+        bounds = v.get_bounds(None)
+        if bounds is not None:
+            axes = self._parse_axes(axes)
+            bounds.flatten(axes, inplace=True)
+        
+        if inplace:
+            v = None
+        return v
+
+
     def floor(self, bounds=True, inplace=False, i=False):
         '''Floor the data array, element-wise.
 
@@ -2044,87 +2092,86 @@ ValueError: Can't take the logarithm to the base 2.718281828459045 of <Units: >
         return v
     #--- End: def
 
-    # 1
+
     def squeeze(self, axes=None, inplace=False, i=False):
         '''Remove size 1 dimensions from the data array
 
-.. seealso:: `insert_dimension`, `flip`, `transpose`
-
-:Parameters:
-
-    axes: (sequence of) `int`, optional
-        The size 1 axes to remove. By default, all size 1 axes are
-        removed. Size 1 axes for removal are identified by their
-        integer positions in the data array.
+    .. seealso:: `insert_dimension`, `flip`, `transpose`
     
-
-    inplace: `bool`, optional
-        If True then do the operation in-place and return `None`.
-
-    i: deprecated at version 3.0.0
-        Use *inplace* parameter instead.
-
-:Returns:
-
-        The construct with squeezed data. If the operation was
-        in-place then `None` is returned.
-
-**Examples:**
-
-
-TODO
-
->>> f.squeeze()
-
->>> f.squeeze(1)
-
->>> f.squeeze([2, -1])
+    :Parameters:
+    
+        axes: (sequence of) `int`, optional
+            The size 1 axes to remove. By default, all size 1 axes are
+            removed. Size 1 axes for removal are identified by their
+            integer positions in the data array.
+        
+    
+        inplace: `bool`, optional
+            If True then do the operation in-place and return `None`.
+    
+        i: deprecated at version 3.0.0
+            Use *inplace* parameter instead.
+    
+    :Returns:
+    
+            The construct with squeezed data. If the operation was
+            in-place then `None` is returned.
+    
+    **Examples:**
+    
+    
+    TODO
+    
+    >>> f.squeeze()
+    
+    >>> f.squeeze(1)
+    
+    >>> f.squeeze([2, -1])
 
         '''
         if i:
             _DEPRECATION_ERROR_KWARGS(self, 'squeeze', i=True) # pragma: no cover
 
         return super().squeeze(axes=axes, inplace=inplace)
-    #--- End: def
     
-    # 1
+
     def trunc(self, bounds=True, inplace=False, i=False):
         '''Truncate the data, element-wise.
 
-The truncated value of the scalar ``x``, is the nearest integer ``i``
-which is closer to zero than ``x`` is. I.e. the fractional part of the
-signed number ``x`` is discarded.
-
-.. versionadded:: 1.0
-
-.. seealso:: `ceil`, `floor`, `rint`
-
-:Parameters:
-
-    bounds: `bool`, optional
-        If False then do not alter any bounds. By default any bounds
-        are also altered.
-
-    inplace: `bool`, optional
-        If True then do the operation in-place and return `None`.
-
-    i: deprecated at version 3.0.0
-        Use *inplace* parameter instead.
-
-:Returns:
-
-        The construct with truncated data. If the operation was
-        in-place then `None` is returned.
-
-**Examples:**
-
->>> print(f.array)
-[-1.9 -1.5 -1.1 -1.   0.   1.   1.1  1.5  1.9]
->>> print(f.trunc().array)
-[-1. -1. -1. -1.  0.  1.  1.  1.  1.]
->>> f.trunc(inplace=True)
->>> print(f.array)
-[-1. -1. -1. -1.  0.  1.  1.  1.  1.]
+    The truncated value of the scalar ``x``, is the nearest integer
+    ``i`` which is closer to zero than ``x`` is. I.e. the fractional
+    part of the signed number ``x`` is discarded.
+    
+    .. versionadded:: 1.0
+    
+    .. seealso:: `ceil`, `floor`, `rint`
+    
+    :Parameters:
+    
+        bounds: `bool`, optional
+            If False then do not alter any bounds. By default any
+            bounds are also altered.
+    
+        inplace: `bool`, optional
+            If True then do the operation in-place and return `None`.
+    
+        i: deprecated at version 3.0.0
+            Use *inplace* parameter instead.
+    
+    :Returns:
+    
+            The construct with truncated data. If the operation was
+            in-place then `None` is returned.
+    
+    **Examples:**
+    
+    >>> print(f.array)
+    [-1.9 -1.5 -1.1 -1.   0.   1.   1.1  1.5  1.9]
+    >>> print(f.trunc().array)
+    [-1. -1. -1. -1.  0.  1.  1.  1.  1.]
+    >>> f.trunc(inplace=True)
+    >>> print(f.array)
+    [-1. -1. -1. -1.  0.  1.  1.  1.  1.]
 
         '''
         if i:
@@ -2143,52 +2190,57 @@ signed number ``x`` is discarded.
         if inplace:
             v = None
         return v
-    #--- End: def
+
     
     def identities(self):
         '''Return all possible identities.
 
-The identities comprise:
+    The identities comprise:
+    
+    * The "standard_name" property.
+    * The "id" attribute, preceeded by ``'id%'``.
+    * The "cf_role" property, preceeded by ``'cf_role='``.
+    * The "axis" property, preceeded by ``'axis='``.
+    * The "long_name" property, preceeded by ``'long_name='``.
+    * All other properties (including "standard_name"), preceeded by
+      the property name and an ``'='``.
+    * The coordinate type (``'X'``, ``'Y'``, ``'Z'`` or ``'T'``).
+    * The netCDF variable name, preceeded by ``'ncvar%'``.
+    
+    The identities of the bounds, if present, are included (with the
+    exception of the bounds netCDF variable name).
 
-* The "standard_name" property.
-* All properties, preceeded by the property name and a colon,
-  e.g. ``'long_name:Air temperature'``.
-* The netCDF variable name, preceeded by ``'ncvar%'``.
-* The coordinate type (one or none of ``'X'``, ``'Y'``, ``'Z'``,
-  ``'T'``).
-* The identities of the bounds, if any.
-
-.. versionadded:: 3.0.0
-
-.. seealso:: `identity`
-
-:Returns:
-
-    `list`
-        The identities.
-
-**Examples:**
-
->>> f.properties()
-{'foo': 'bar',
- 'long_name': 'Air Temperature',
- 'standard_name': 'air_temperature'}
->>> f.nc_get_variable()
-'tas'
->>> f.identities()
-['air_temperature',
- 'long_name=Air Temperature',
- 'foo=bar',
- 'standard_name=air_temperature',
- 'ncvar%tas']
-
->>> f.properties()
-{}
->>> f.bounds.properties()
-{'axis': 'Z',
- 'units': 'm'}
->>> f.identities()
-['axis=Z', 'units=m', 'ncvar%z']
+    .. versionadded:: 3.0.0
+    
+    .. seealso:: `id`, `identity`
+    
+    :Returns:
+    
+        `list`
+            The identities.
+    
+    **Examples:**
+    
+    >>> f.properties()
+    {'foo': 'bar',
+     'long_name': 'Air Temperature',
+     'standard_name': 'air_temperature'}
+    >>> f.nc_get_variable()
+    'tas'
+    >>> f.identities()
+    ['air_temperature',
+     'long_name=Air Temperature',
+     'foo=bar',
+     'standard_name=air_temperature',
+     'ncvar%tas']
+    
+    >>> f.properties()
+    {}
+    >>> f.bounds.properties()
+    {'axis': 'Z',
+     'units': 'm'}
+    >>> f.identities()
+    ['axis=Z', 'units=m', 'ncvar%z']
 
         '''
         identities = super().identities()
@@ -2200,25 +2252,28 @@ The identities comprise:
 # TODO ncvar AND?
 
         return identities
-    #--- End: def
 
 
-    def identity(self, default='', strict=False, nc_only=False,
-                 relaxed_identity=None):
+    def identity(self, default='', strict=False, relaxed=False,
+                 nc_only=False, relaxed_identity=None):
         '''Return the canonical identity.
 
     By default the identity is the first found of the following:
     
-    1. The "standard_name" property.
-    2. The "id" attribute, preceeded by ``'id%='``.
-    3. The "cf_role" property, preceeded by ``'cf_role='``.
-    4. The "axis" property, preceeded by ``'axis='``.
-    5. The "long_name" property, preceeded by ``'long_name='``.
-    6. The netCDF variable name, preceeded by ``'ncvar%'``.
-    7. The identity of the bounds, if any.
-    8. The value of the *default* parameter.
+    * The "standard_name" property.
+    * The "id" attribute, preceeded by ``'id%'``.
+    * The "cf_role" property, preceeded by ``'cf_role='``.
+    * The "axis" property, preceeded by ``'axis='``.
+    * The "long_name" property, preceeded by ``'long_name='``.
+    * The netCDF variable name, preceeded by ``'ncvar%'``.
+    * The coordinate type (``'X'``, ``'Y'``, ``'Z'`` or ``'T'``).
+    * The value of the *default* parameter.
+
+    If no identity can be found on the construct then the identity is
+    taken from the bounds, if present (with the exception of the
+    bounds netCDF variable name).
     
-    .. seealso:: `identities`
+    .. seealso:: `id`, `identities`
     
     :Parameters:
     
@@ -2226,6 +2281,21 @@ The identities comprise:
             If no identity can be found then return the value of the
             default parameter.
     
+        strict: `bool`, optional 
+            If True then only take the identity from the
+            "standard_name" property or the "id" attribute, in that
+            order.
+
+        relaxed: `bool`, optional
+            If True then only take the identity from the
+            "standard_name" property, the "id" attribute, the
+            "long_name" property or netCDF variable name, in that
+            order.
+
+        nc_only: `bool`, optional       
+            If True then only take the identity from the netCDF
+            variable name.
+
     :Returns:
     
             The identity.
@@ -2270,14 +2340,21 @@ The identities comprise:
         if relaxed_identity:
             _DEPRECATAION_ERROR_KWARGS(self, 'identity', relaxed_identity=True) # pragma: no cover
             
-        identity = super().identity(default=None, strict=strict, nc_only=nc_only)
+        identity = super().identity(default=None, strict=strict,
+                                    relaxed=relaxed, nc_only=nc_only)
+
         if identity is not None:
             return identity
 
         bounds = self.get_bounds(None)
         if bounds is not None:
-            return bounds.identity(default=default, strict=strict, nc_only=nc_only)
-        
+            out = bounds.identity(default=None, strict=strict,
+                                  relaxed=relaxed, nc_only=nc_only)
+
+            if out is not None and not out.startswith('ncvar%'):
+                return out
+        #--- End: if
+                
         return default
 
 
@@ -2294,40 +2371,39 @@ The identities comprise:
         print(cf_inspect(self)) # pragma: no cover
 
 
-    # 1
     def rint(self, bounds=True, inplace=False, i=False):
         '''Round the data to the nearest integer, element-wise.
 
-.. versionadded:: 1.0
-
-.. seealso:: `ceil`, `floor`, `trunc`
-
-:Parameters:
-
-    bounds: `bool`, optional
-        If False then do not alter any bounds. By default any bounds
-        are also altered.
-
-    inplace: `bool`, optional
-        If True then do the operation in-place and return `None`.
-
-    i: deprecated at version 3.0.0
-        Use *inplace* parameter instead.
-
-:Returns:
-        
-        The construct with rounded data. If the operation was in-place
-        then `None` is returned.
-
-**Examples:**
-
->>> print(f.array)
-[-1.9 -1.5 -1.1 -1.   0.   1.   1.1  1.5  1.9]
->>> print(f.rint().array)
-[-2. -2. -1. -1.  0.  1.  1.  2.  2.]
->>> f.rint(inplace=True)
->>> print(f.array)
-[-2. -2. -1. -1.  0.  1.  1.  2.  2.]
+    .. versionadded:: 1.0
+    
+    .. seealso:: `ceil`, `floor`, `trunc`
+    
+    :Parameters:
+    
+        bounds: `bool`, optional
+            If False then do not alter any bounds. By default any
+            bounds are also altered.
+    
+        inplace: `bool`, optional
+            If True then do the operation in-place and return `None`.
+    
+        i: deprecated at version 3.0.0
+            Use *inplace* parameter instead.
+    
+    :Returns:
+            
+            The construct with rounded data. If the operation was
+            in-place then `None` is returned.
+    
+    **Examples:**
+    
+    >>> print(f.array)
+    [-1.9 -1.5 -1.1 -1.   0.   1.   1.1  1.5  1.9]
+    >>> print(f.rint().array)
+    [-2. -2. -1. -1.  0.  1.  1.  2.  2.]
+    >>> f.rint(inplace=True)
+    >>> print(f.array)
+    [-2. -2. -1. -1.  0.  1.  1.  2.  2.]
 
         '''
         if i:
@@ -2346,57 +2422,56 @@ The identities comprise:
         if inplace:
             v = None            
         return v
-    #--- End: def
 
-    # 1
+
     def round(self, decimals=0, bounds=True, inplace=False, i=False):
         '''Round the data to the given number of decimals.
 
-Data elements are evenly rounded to the given number of decimals.
-
-.. note:: Values exactly halfway between rounded decimal values are
-          rounded to the nearest even value. Thus 1.5 and 2.5 round to
-          2.0, -0.5 and 0.5 round to 0.0, etc. Results may also be
-          surprising due to the inexact representation of decimal
-          fractions in the IEEE floating point standard and errors
-          introduced when scaling by powers of ten.
- 
-.. versionadded:: 1.1.4
-
-.. seealso:: `ceil`, `floor`, `rint`, `trunc`
-
-:Parameters:
-	
-    decimals: `int`, optional
-        Number of decimal places to round to (0 by default). If
-        decimals is negative, it specifies the number of positions to
-        the left of the decimal point.
-
-    bounds: `bool`, optional
-        If False then do not alter any bounds. By default any bounds
-        are also altered.
-
-    inplace: `bool`, optional
-        If True then do the operation in-place and return `None`.
-
-    i: deprecated at version 3.0.0
-        Use *inplace* parameter instead.
-
-:Returns:
-
-        The construct with rounded data. If the operation was in-place
-        then `None` is returned.
-
-**Examples:**
-
->>> print(f.array)
-[-1.81, -1.41, -1.01, -0.91,  0.09,  1.09,  1.19,  1.59,  1.99])
->>> print(f.round().array)
-[-2., -1., -1., -1.,  0.,  1.,  1.,  2.,  2.]
->>> print(f.round(1).array)
-[-1.8, -1.4, -1. , -0.9,  0.1,  1.1,  1.2,  1.6,  2. ]
->>> print(f.round(-1).array)
-[-0., -0., -0., -0.,  0.,  0.,  0.,  0.,  0.]
+    Data elements are evenly rounded to the given number of decimals.
+    
+    .. note:: Values exactly halfway between rounded decimal values
+              are rounded to the nearest even value. Thus 1.5 and 2.5
+              round to 2.0, -0.5 and 0.5 round to 0.0, etc. Results
+              may also be surprising due to the inexact representation
+              of decimal fractions in the IEEE floating point standard
+              and errors introduced when scaling by powers of ten.
+     
+    .. versionadded:: 1.1.4
+    
+    .. seealso:: `ceil`, `floor`, `rint`, `trunc`
+    
+    :Parameters:
+    	
+        decimals: `int`, optional
+            Number of decimal places to round to (0 by default). If
+            decimals is negative, it specifies the number of positions
+            to the left of the decimal point.
+    
+        bounds: `bool`, optional
+            If False then do not alter any bounds. By default any
+            bounds are also altered.
+    
+        inplace: `bool`, optional
+            If True then do the operation in-place and return `None`.
+    
+        i: deprecated at version 3.0.0
+            Use *inplace* parameter instead.
+    
+    :Returns:
+    
+            The construct with rounded data. If the operation was
+            in-place then `None` is returned.
+    
+    **Examples:**
+    
+    >>> print(f.array)
+    [-1.81, -1.41, -1.01, -0.91,  0.09,  1.09,  1.19,  1.59,  1.99])
+    >>> print(f.round().array)
+    [-2., -1., -1., -1.,  0.,  1.,  1.,  2.,  2.]
+    >>> print(f.round(1).array)
+    [-1.8, -1.4, -1. , -0.9,  0.1,  1.1,  1.2,  1.6,  2. ]
+    >>> print(f.round(-1).array)
+    [-0., -0., -0., -0.,  0.,  0.,  0.,  0.,  0.]
 
         '''
         if i:
@@ -2416,32 +2491,32 @@ Data elements are evenly rounded to the given number of decimals.
         if inplace:
             v = None
         return v
-    #--- End: def
 
-    # 0
+
     def roll(self, iaxis, shift, inplace=False, i=False):
         '''Roll the data along an axis.
 
-.. seealso:: `insert_dimension`, `flip`, `squeeze`, `transpose`
+    .. seealso:: `insert_dimension`, `flip`, `squeeze`, `transpose`
+    
+    :Parameters:
+    
+        iaxis: `int`
+            TODO
+            
+        inplace: `bool`, optional
+            If True then do the operation in-place and return `None`.
+    
+        i: deprecated at version 3.0.0
+            Use *inplace* parmaeter instead.
+    
+    :Returns:
+    
+    TODO
+    
+    **Examples:**
+    
+    TODO
 
-:Parameters:
-
-    iaxis: `int`
-        TODO
-        
-    inplace: `bool`, optional
-        If True then do the operation in-place and return `None`.
-
-    i: deprecated at version 3.0.0
-        Use *inplace* parmaeter instead.
-
-:Returns:
-
-TODO
-
-**Examples:**
-
-TODO
         '''
         if i:
             _DEPRECATION_ERROR_KWARGS(self, 'roll', i=True) # pragma: no cover
@@ -2459,10 +2534,10 @@ TODO
         if inplace:
             v = None
         return v
-    #--- End: def
+
 
     # ----------------------------------------------------------------
-    # Deprecated attributes
+    # Deprecated attributes and methods
     # ----------------------------------------------------------------
     @property
     def hasbounds(self):
@@ -2472,20 +2547,18 @@ TODO
         _DEPRECATION_ERROR_ATTRIBUTE(
             self, 'hasbounds',
             "Use method 'has_bounds' instead.") # pragma: no cover
-    #--- End: def
-    
-    # ----------------------------------------------------------------
-    # Deprecated methods
-    # ----------------------------------------------------------------
+
+        
     def expand_dims(self, position=0, i=False):
         '''Insert a size 1 axis into the data array.
 
-Deprecated at version 3.0.0. Use method 'insert_dimension' instead.
+    Deprecated at version 3.0.0. Use method 'insert_dimension'
+    instead.
 
         '''
         _DEPRECATION_ERROR_METHOD(
             self, 'expand_dims',
             "Use method 'insert_dimension' instead.") # pragma: no cover
-    #--- End: def
 
+        
 #--- End: class
