@@ -34,7 +34,6 @@ class FieldTest(unittest.TestCase):
 #        self.test_only = ['NOTHING!!!!']
 #        self.test_only = ['test_Field__add__']
 #        self.test_only = ['test_Field_cumsum']
-#        self.test_only = ['test_Field_flatten']
 #        self.test_only = ['test_Field_collapse']
 #        self.test_only = ['test_Field_radius']
 #        self.test_only = ['test_Field_field_ancillary']
@@ -47,27 +46,46 @@ class FieldTest(unittest.TestCase):
 #        self.test_only = ['test_Field_autocyclic']
 #        self.test_only = ['test_Field_anchor']
 #        self.test_only = ['test_Field_mask_invalid']
-#        self.test_only = ['test_Field_item']
+#        self.test_only = ['test_Field_test_Field_creation_commands']
 #        self.test_only = ['test_Field_section']
 #        self.test_only = ['test_Field_flip']
 #        self.test_only = ['test_Field_Field_domain_mask']
 #        self.test_only = ['test_Field_bin']
 
+    def test_Field_example_field(self):
+        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+            return
 
+        for n in (1, 2):
+            f = cf.Field.example_field(n)
+            
+        
+    def test_Field_creation_commands(self):
+        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+            return
+
+        f = self.f.copy()
+
+        for rd in (False, True):
+            for indent in (0, 4):
+                for s in (False, True):
+                    _ = f.creation_commands(representative_data=rd,
+                                            indent=indent,
+                                            string=s)
+        #--- End: for
+            
+        
     def test_Field_flatten(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
         
         f = self.f.copy()
-#        print(f)
-#        print(f.constructs)
+
         axis = f.set_construct(cf.DomainAxis(1))
         d = cf.DimensionCoordinate()
         d.standard_name = 'time'
         d.set_data(cf.Data(123., 'days since 2000-01-02'))
         f.set_construct(d, axes=axis)
-#        print(f)
-#        print(f.constructs)
 
         g = f.flatten()
         h = f.flatten(list(range(f.ndim)))
@@ -83,9 +101,7 @@ class FieldTest(unittest.TestCase):
             self.assertTrue(g.equals(f, verbose=True))
 
         for axes in axes_combinations(f):
-#            print (axes)
             g = f.flatten(axes)
-#            print (g)
 
             if len(axes) <= 1:
                 shape  = f.shape
@@ -246,40 +262,88 @@ class FieldTest(unittest.TestCase):
         f = self.f.copy()
         f[0, 3] *= -1
         f[0, 5, ::2] = cf.masked
-
-        for method in ('sum', 'min', 'max', 'maximum_absolute_value',
-                       'maximum_absolute_value', 'mid_range', 'range',
-                       'sample_size', 'sum_of_squares', 'mean',
-                       'mean_absolute_value',
-                       'root_mean_square', 'integral', 'var', 'sd'):
-            a = f.collapse(method).data
-            b = getattr(f.data, method)()
-            self.assertTrue(a.equals(b, verbose=True),
-                            '{} unweighted {!r}, {!r}'.format(method, a, b))
-
-        for method in ('sum', 'min', 'max', 'maximum_absolute_value',
-                       'maximum_absolute_value', 'mid_range', 'range',
-                       'sample_size', 'sum_of_squares'):
-            a = f.collapse(method, weights='area').data
-            b = getattr(f.data, method)()
-            self.assertTrue(a.equals(b, verbose=True),
-                            '{} weighted, {!r}, {!r}'.format(method, a, b))
-
-        for method in ('mean', 'mean_absolute_value',
-                       'root_mean_square', 'var', 'sd'):
-            weights = f.weights('area', components=True)
-            a = f.collapse(method, weights='area').data
-            b = getattr(f.data, method)(weights=weights)
-            self.assertTrue(a.equals(b, verbose=True),
-                            '{} weighted, {!r}, {!r}'.format(method, a, b))
+        
+        for axes in axes_combinations(f):
+            for method in ('sum',
+                           'min',
+                           'max',
+                           'minimum_absolute_value',
+                           'maximum_absolute_value',
+                           'mid_range',
+                           'range',
+                           'sample_size',
+                           'sum_of_squares',
+                           'median',
+                           'sum_of_weights',
+                           'sum_of_weights2',):
+                for weights in (None, 'area'):
+#                    print(method, axes, weights)
+                    a = f.collapse(method, axes=axes, weights=weights).data
+                    b = getattr(f.data, method)(axes=axes)
+                    self.assertTrue(a.equals(b, rtol=1e-05, atol=1e-08, verbose=True),
+                                    '{} weights={}, axes={}, {!r}, {!r}'.format(method, weights, axes, a, b))
+            #--- End: for
             
-        for method in ('integral',):
-            weights = f.weights('area', components=True, measure=True)
-            a = f.collapse(method, weights='area', measure=True).data
-            b = getattr(f.data, method)(weights=weights)
-            self.assertTrue(a.equals(b, verbose=True),
-                            '{} weighted, {!r}, {!r}'.format(method, a, b))
+            for method in ('mean',
+                           'mean_absolute_value',
+#                           'mean_of_upper_decile',
+                           'root_mean_square'):
+                for weights in (None, 'area'):
+#                    print(method, axes, weights)
+                    if weights is not None:
+                        d_weights = f.weights(weights, components=True)
+                    else:
+                        d_weights = weights
+                        
+                    a = f.collapse(method, axes=axes, weights=weights).data
+                    b = getattr(f.data, method)(axes=axes, weights=d_weights)
+                    self.assertTrue(a.equals(b, rtol=1e-05, atol=1e-08, verbose=True),
+                                    '{} weights={}, axes={}, {!r}, {!r}'.format(method, weights, axes, a, b))
+                    #--- End: for
             
+            for method in ('integral',):
+                weights = 'area'
+#                print(method, axes, weights)
+                d_weights = f.weights(weights, components=True, measure=True)               
+                a = f.collapse(method, axes=axes, weights=weights, measure=True).data
+                b = getattr(f.data, method)(axes=axes, weights=d_weights)
+                self.assertTrue(a.equals(b, rtol=1e-05, atol=1e-08, verbose=True),
+                                '{} weighted axes={}, {!r}, {!r}'.format(method, axes, a, b))
+        #--- End: for
+        
+        for axes in axes_combinations(f):
+            if axes == (0,):
+                continue
+            
+            for method in ('var',
+                           'sd'):
+                for weights in (None, 'area'):            
+#                    print(method, axes, weights)
+                    if weights is not None:
+                        d_weights = f.weights(weights, components=True)
+                    else:
+                        d_weights = None
+
+                    a = f.collapse(method, axes=axes, weights=weights).data
+                    b = getattr(f.data, method)(axes=axes, ddof=1, weights=d_weights)
+                    self.assertTrue(a.equals(b, rtol=1e-05, atol=1e-08, verbose=True),
+                                    '{} weights={}, axes={}, {!r}, {!r}'.format(method, weights, axes, a, b))
+            #--- End: for
+            
+            for method in ('mean_of_upper_decile',):
+                for weights in (None, 'area'):            
+#                    print(method, axes, weights)
+                    if weights is not None:
+                        d_weights = f.weights(weights, components=True)
+                    else:
+                        d_weights = None
+                        
+                    a = f.collapse(method, axes=axes, weights=weights).data
+                    b = getattr(f.data, method)(axes=axes, weights=d_weights)
+                    self.assertTrue(a.equals(b, rtol=1e-05, atol=1e-08, verbose=True),
+                                    '{} weights={}, axes={}, {!r}, {!r}'.format(method, weights, axes, a, b))
+        #--- End: for
+        
 
     def test_Field_all(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
@@ -380,21 +444,21 @@ class FieldTest(unittest.TestCase):
         ac = numpy.ma.masked_all((3, 7))
         ac[0, 0:5] = [1.0, 2.0, 3.0, -99, 5.0]
         ac[0, 3  ] = numpy.ma.masked
-        ac[1, 1:5] =      [1.5, 2.5, 3.5, 4.5]
-        ac[2, 3:7] =                [1.0, 2.0, 3.0, 5.0]
+        ac[1, 1:5] = [1.5, 2.5, 3.5, 4.5]
+        ac[2, 3:7] = [1.0, 2.0, 3.0, 5.0]
         
         ae = numpy.ma.masked_all((3, 8))
         ae[0, 0:5] = [1.0, 2.0, 3.0, -99, 5.0]
         ae[0, 3  ] = numpy.ma.masked
-        ae[1, 1:5] =      [1.5, 2.5, 3.5, 4.5]
-        ae[2, 3:8] =                [1.0, 2.0, 3.0, -99, 5.0]
+        ae[1, 1:5] = [1.5, 2.5, 3.5, 4.5]
+        ae[2, 3:8] = [1.0, 2.0, 3.0, -99, 5.0]
         ae[2, 6  ] = numpy.ma.masked
 
         af = numpy.ma.masked_all((4, 9))
         af[1, 0:5] = [1.0, 2.0, 3.0, -99, 5.0]
         af[1, 3  ] = numpy.ma.masked
-        af[2, 1:5] =      [1.5, 2.5, 3.5, 4.5]
-        af[3, 3:8] =                [1.0, 2.0, 3.0, -99, 5.0]
+        af[2, 1:5] = [1.5, 2.5, 3.5, 4.5]
+        af[3, 3:8] = [1.0, 2.0, 3.0, -99, 5.0]
         af[3, 6  ] = numpy.ma.masked
         
         query1 = cf.wi(1, 5) & cf.ne(4)
@@ -407,10 +471,9 @@ class FieldTest(unittest.TestCase):
                                           [ac.shape, ae.shape, af.shape],
                                           [ac, ae, af]):
                 message = 'method={!r}'.format(method)
-
                 g = f.subspace(method, time=query1)
                 t = g.coordinate('time')
-
+                
                 self.assertTrue(g.shape == shape, message)
                 self.assertTrue(t.shape == shape, message)
 
@@ -1589,6 +1652,15 @@ class FieldTest(unittest.TestCase):
         c = f.auxiliary_coordinates(identities[0])
         self.assertTrue(f.auxs(identities[0]).equals(c, verbose=True))
 
+        self.assertTrue(f.aux('long_name=qwerty:asd', None) is None)
+        self.assertTrue(len(f.auxs('long_name=qwerty:asd')) == 0)
+
+        with self.assertRaises(Exception):
+            f.aux('long_name:qwerty')
+        
+        with self.assertRaises(Exception):
+            f.auxs('long_name:qwerty')
+        
     
     def test_Field_coordinate(self):        
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
@@ -1619,7 +1691,16 @@ class FieldTest(unittest.TestCase):
             c = f.coordinates(identities[0])
             self.assertTrue(f.coords(identities[0]).equals(c, verbose=True))
 
-    
+        self.assertTrue(f.coord('long_name=qwerty:asd', None) is None)
+        self.assertTrue(len(f.coords('long_name=qwerty:asd')) == 0)
+        
+        with self.assertRaises(Exception):
+            f.coord('long_name:qwerty')
+            
+        with self.assertRaises(Exception):
+            f.coords('long_name:qwerty')
+
+            
     def test_Field_coordinate_reference(self):        
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
@@ -1718,6 +1799,15 @@ class FieldTest(unittest.TestCase):
             c = f.dimension_coordinates(identities[0])
             self.assertTrue(f.dims(identities[0]).equals(c, verbose=True))
 
+        self.assertTrue(f.dim('long_name=qwerty:asd', None) is None)
+        self.assertTrue(len(f.dims('long_name=qwerty:asd')) == 0)
+
+        with self.assertRaises(Exception):
+            f.dim('long_name:qwerty')
+        
+        with self.assertRaises(Exception):
+            f.dims('long_name:qwerty')
+        
             
     def test_Field_cell_measure(self):        
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
@@ -1739,6 +1829,15 @@ class FieldTest(unittest.TestCase):
         self.assertTrue(len(f.measures('measure:area')) == 1)
         self.assertTrue(len(f.measures(*['measure:area'])) == 1)
 
+        self.assertTrue(f.measure('long_name=qwerty:asd', None) is None)
+        self.assertTrue(len(f.measures('long_name=qwerty:asd')) == 0)
+
+        with self.assertRaises(Exception):
+            f.measure('long_name:qwerty')
+        
+        with self.assertRaises(Exception):
+            f.measures('long_name:qwerty')
+        
     
     def test_Field_cell_method(self):        
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
@@ -1778,6 +1877,15 @@ class FieldTest(unittest.TestCase):
         c = f.domain_ancillaries(identities[0])
         self.assertTrue(f.domain_ancs(identities[0]).equals(c, verbose=True))
 
+        self.assertTrue(f.domain_anc('long_name=qwerty:asd', None) is None)
+        self.assertTrue(len(f.domain_ancs('long_name=qwerty:asd')) == 0)
+
+        with self.assertRaises(Exception):
+            f.domain_anc('long_name:qwerty')
+        
+        with self.assertRaises(Exception):
+            f.domain_ancs('long_name:qwerty')
+        
     
     def test_Field_field_ancillary(self):        
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
@@ -1807,7 +1915,16 @@ class FieldTest(unittest.TestCase):
         c = f.field_ancillaries(identities[0])
         self.assertTrue(f.field_ancs(identities[0]).equals(c, verbose=True))
 
-    
+        self.assertTrue(f.field_anc('long_name=qwerty:asd', None) is None)
+        self.assertTrue(len(f.field_ancs('long_name=qwerty:asd')) == 0)
+
+        with self.assertRaises(Exception):
+            f.field_anc('long_name:qwerty')
+        
+        with self.assertRaises(Exception):
+            f.field_ancs('long_name:qwerty')
+        
+   
     def test_Field_transpose(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
